@@ -1204,10 +1204,14 @@ gst_jpeg_dec_handle_frame (GstVideoDecoder * bdec, GstVideoCodecFrame * frame)
   guint8 *data;
   gsize nbytes;
 
-  gst_buffer_map (frame->input_buffer, &dec->current_frame_map, GST_MAP_READ);
+  if (!gst_buffer_map (frame->input_buffer, &dec->current_frame_map,
+          GST_MAP_READ))
+    goto map_failed;
 
   data = dec->current_frame_map.data;
   nbytes = dec->current_frame_map.size;
+  if (nbytes < 2)
+    goto need_more_data;
   has_eoi = ((data[nbytes - 2] == 0xff) && (data[nbytes - 1] == 0xd9));
 
   /* some cameras fail to send an end-of-image marker (EOI),
@@ -1393,6 +1397,13 @@ need_more_data:
     goto exit;
   }
   /* ERRORS */
+map_failed:
+  {
+    GST_ELEMENT_ERROR (dec, RESOURCE, READ, (_("Failed to read memory")),
+        ("gst_buffer_map() failed for READ access"));
+    ret = GST_FLOW_ERROR;
+    goto exit;
+  }
 decode_error:
   {
     gchar err_msg[JMSG_LENGTH_MAX];
