@@ -58,6 +58,8 @@ struct _GstJpegEnc
   GstVideoCodecFrame *current_frame;
   GstFlowReturn res;
 
+  gboolean input_caps_changed;
+
   guint channels;
 
   gint inc[GST_VIDEO_MAX_COMPONENTS];
@@ -84,6 +86,7 @@ struct _GstJpegEnc
   gint quality;
   gint smoothing;
   gint idct_method;
+  gboolean snapshot;
 
   GstMemory *output_mem;
   GstMapInfo output_map;
