@@ -88,7 +88,10 @@ GST_DEBUG_CATEGORY_STATIC (gst_vpxenc_debug);
 #define DEFAULT_DEADLINE VPX_DL_BEST_QUALITY
 #define DEFAULT_NOISE_SENSITIVITY 0
 #define DEFAULT_SHARPNESS 0
-#define DEFAULT_STATIC_THRESHOLD 0
+
+/* Use same default value as Chromium/webrtc. */
+#define DEFAULT_STATIC_THRESHOLD 1
+
 #define DEFAULT_TOKEN_PARTITIONS 0
 #define DEFAULT_ARNR_MAXFRAMES 0
 #define DEFAULT_ARNR_STRENGTH 3
@@ -582,8 +585,8 @@ gst_vpx_enc_class_init (GstVPXEncClass * klass)
 
   g_object_class_install_property (gobject_class, PROP_STATIC_THRESHOLD,
       g_param_spec_int ("static-threshold", "Static Threshold",
-          "Motion detection threshold",
-          0, G_MAXINT, DEFAULT_STATIC_THRESHOLD,
+          "Motion detection threshold. Recommendation is to set 100 for "
+          "screen/window sharing", 0, G_MAXINT, DEFAULT_STATIC_THRESHOLD,
           (GParamFlags) (G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)));
 
   g_object_class_install_property (gobject_class, PROP_TOKEN_PARTITIONS,
@@ -1668,6 +1671,7 @@ gst_vpx_enc_process (GstVPXEnc * encoder)
   GstVideoCodecFrame *frame;
   GstFlowReturn ret = GST_FLOW_OK;
   GstVPXEncClass *vpx_enc_class;
+  vpx_codec_pts_t pts;
 
   video_encoder = GST_VIDEO_ENCODER (encoder);
   vpx_enc_class = GST_VPX_ENC_GET_CLASS (encoder);
@@ -1707,7 +1711,21 @@ gst_vpx_enc_process (GstVPXEnc * encoder)
     }
 
     invisible = (pkt->data.frame.flags & VPX_FRAME_IS_INVISIBLE) != 0;
-    frame = gst_video_encoder_get_oldest_frame (video_encoder);
+
+    /* discard older frames that were dropped by libvpx */
+    frame = NULL;
+    do {
+      if (frame)
+        gst_video_encoder_finish_frame (video_encoder, frame);
+      frame = gst_video_encoder_get_oldest_frame (video_encoder);
+      pts =
+          gst_util_uint64_scale (frame->pts,
+          encoder->cfg.g_timebase.den,
+          encoder->cfg.g_timebase.num * (GstClockTime) GST_SECOND);
+      GST_TRACE_OBJECT (encoder, "vpx pts: %" G_GINT64_FORMAT
+          ", gst frame pts: %" G_GINT64_FORMAT, pkt->data.frame.pts, pts);
+    } while (pkt->data.frame.pts > pts);
+
     g_assert (frame != NULL);
     if ((pkt->data.frame.flags & VPX_FRAME_IS_KEY) != 0)
       GST_VIDEO_CODEC_FRAME_SET_SYNC_POINT (frame);
