@@ -105,10 +105,8 @@ gst_wavpack_dec_class_init (GstWavpackDecClass * klass)
   GstElementClass *element_class = (GstElementClass *) (klass);
   GstAudioDecoderClass *base_class = (GstAudioDecoderClass *) (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class, &sink_factory);
   gst_element_class_set_static_metadata (element_class, "Wavpack audio decoder",
       "Codec/Decoder/Audio",
       "Decodes Wavpack audio data",
