@@ -24,7 +24,7 @@
  * @see_also: goom, synaesthesia
  *
  * Goom2k1 is an audio visualisation element. It creates warping structures
- * based on the incomming audio signal. Goom2k1 is the older version of the
+ * based on the incoming audio signal. Goom2k1 is the older version of the
  * visualisation. Also available is goom2k4, with a different look.
  *
  * <refsect2>
@@ -104,10 +104,8 @@ gst_goom2k1_class_init (GstGoom2k1Class * klass)
       "GOOM: what a GOOM! 2k1 edition", "Visualization",
       "Takes frames of data and outputs video frames using the GOOM 2k1 filter",
       "Wim Taymans <wim@fluendo.com>");
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
 
   GST_DEBUG_CATEGORY_INIT (goom2k1_debug, "goom2k1", 0,
       "goom2k1 visualisation element");
