@@ -45,6 +45,14 @@
 
 #include "gstimagefreeze.h"
 
+#define DEFAULT_NUM_BUFFERS     -1
+
+enum
+{
+  PROP_0,
+  PROP_NUM_BUFFERS
+};
+
 static void gst_image_freeze_finalize (GObject * object);
 
 static void gst_image_freeze_reset (GstImageFreeze * self);
@@ -52,6 +60,10 @@ static void gst_image_freeze_reset (GstImageFreeze * self);
 static GstStateChangeReturn gst_image_freeze_change_state (GstElement * element,
     GstStateChange transition);
 
+static void gst_image_freeze_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec);
+static void gst_image_freeze_get_property (GObject * object, guint prop_id,
+    GValue * value, GParamSpec * pspec);
 static GstFlowReturn gst_image_freeze_sink_chain (GstPad * pad,
     GstObject * parent, GstBuffer * buffer);
 static gboolean gst_image_freeze_sink_event (GstPad * pad, GstObject * parent,
@@ -71,11 +83,11 @@ static gboolean gst_image_freeze_src_query (GstPad * pad, GstObject * parent,
 static GstStaticPadTemplate sink_pad_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw"));
+    GST_STATIC_CAPS ("video/x-raw(ANY)"));
 
 static GstStaticPadTemplate src_pad_template =
 GST_STATIC_PAD_TEMPLATE ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-    GST_STATIC_CAPS ("video/x-raw"));
+    GST_STATIC_CAPS ("video/x-raw(ANY)"));
 
 GST_DEBUG_CATEGORY_STATIC (gst_image_freeze_debug);
 #define GST_CAT_DEFAULT gst_image_freeze_debug
@@ -91,6 +103,14 @@ gst_image_freeze_class_init (GstImageFreezeClass * klass)
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
   gobject_class->finalize = gst_image_freeze_finalize;
+  gobject_class->set_property = gst_image_freeze_set_property;
+  gobject_class->get_property = gst_image_freeze_get_property;
+
+  g_object_class_install_property (gobject_class, PROP_NUM_BUFFERS,
+      g_param_spec_int ("num-buffers", "num-buffers",
+          "Number of buffers to output before sending EOS (-1 = unlimited)",
+          -1, G_MAXINT, DEFAULT_NUM_BUFFERS, G_PARAM_READWRITE |
+          G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state =
       GST_DEBUG_FUNCPTR (gst_image_freeze_change_state);
@@ -101,10 +121,10 @@ gst_image_freeze_class_init (GstImageFreezeClass * klass)
       "Generates a still frame stream from an image",
       "Sebastian Dröge <sebastian.droege@collabora.co.uk>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_pad_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_pad_template));
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &sink_pad_template);
+  gst_element_class_add_static_pad_template (gstelement_class,
+      &src_pad_template);
 }
 
 static void
@@ -130,6 +150,8 @@ gst_image_freeze_init (GstImageFreeze * self)
 
   g_mutex_init (&self->lock);
 
+  self->num_buffers = DEFAULT_NUM_BUFFERS;
+
   gst_image_freeze_reset (self);
 }
 
@@ -138,6 +160,8 @@ gst_image_freeze_finalize (GObject * object)
 {
   GstImageFreeze *self = GST_IMAGE_FREEZE (object);
 
+  self->num_buffers = DEFAULT_NUM_BUFFERS;
+
   gst_image_freeze_reset (self);
 
   g_mutex_clear (&self->lock);
@@ -152,6 +176,7 @@ gst_image_freeze_reset (GstImageFreeze * self)
 
   g_mutex_lock (&self->lock);
   gst_buffer_replace (&self->buffer, NULL);
+  self->num_buffers_left = self->num_buffers;
 
   gst_segment_init (&self->segment, GST_FORMAT_TIME);
   self->need_segment = TRUE;
@@ -265,10 +290,6 @@ gst_image_freeze_sink_getcaps (GstImageFreeze * self, GstCaps * filter)
   GstPad *pad;
 
   pad = self->sinkpad;
-  ret = gst_pad_get_current_caps (pad);
-  if (ret != NULL) {
-    goto done;
-  }
 
   if (filter) {
     filter = gst_caps_copy (filter);
@@ -292,7 +313,6 @@ gst_image_freeze_sink_getcaps (GstImageFreeze * self, GstCaps * filter)
   ret = gst_caps_make_writable (ret);
   gst_image_freeze_remove_fps (self, ret);
 
-done:
   GST_LOG_OBJECT (pad, "Returning caps: %" GST_PTR_FORMAT, ret);
 
   return ret;
@@ -491,6 +511,13 @@ gst_image_freeze_src_query (GstPad * pad, GstObject * parent, GstQuery * query)
       ret = TRUE;
       break;
     }
+    case GST_QUERY_LATENCY:
+      /* This will only return an accurate latency for the first buffer since
+       * all further buffers outputted by us are just copies of that one, and
+       * the latency is 0 in that case. However, latency changes are not
+       * straightforward, so let's do the conservative fix for now. */
+      ret = gst_pad_query_default (pad, parent, query);
+      break;
     default:
       ret = FALSE;
       break;
@@ -571,6 +598,7 @@ gst_image_freeze_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
       gboolean flush;
       guint32 seqnum;
 
+      seqnum = gst_event_get_seqnum (event);
       gst_event_parse_seek (event, &rate, &format, &flags, &start_type, &start,
           &stop_type, &stop);
       gst_event_unref (event);
@@ -598,7 +626,6 @@ gst_image_freeze_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
         }
       }
 
-      seqnum = gst_event_get_seqnum (event);
       if (flush) {
         GstEvent *e;
 
@@ -668,6 +695,42 @@ gst_image_freeze_src_event (GstPad * pad, GstObject * parent, GstEvent * event)
   return ret;
 }
 
+static void
+gst_image_freeze_set_property (GObject * object, guint prop_id,
+    const GValue * value, GParamSpec * pspec)
+{
+  GstImageFreeze *self;
+
+  self = GST_IMAGE_FREEZE (object);
+
+  switch (prop_id) {
+    case PROP_NUM_BUFFERS:
+      self->num_buffers = g_value_get_int (value);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
+static void
+gst_image_freeze_get_property (GObject * object, guint prop_id, GValue * value,
+    GParamSpec * pspec)
+{
+  GstImageFreeze *self;
+
+  self = GST_IMAGE_FREEZE (object);
+
+  switch (prop_id) {
+    case PROP_NUM_BUFFERS:
+      g_value_set_int (value, self->num_buffers);
+      break;
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+      break;
+  }
+}
+
 static GstFlowReturn
 gst_image_freeze_sink_chain (GstPad * pad, GstObject * parent,
     GstBuffer * buffer)
@@ -715,6 +778,18 @@ gst_image_freeze_src_loop (GstPad * pad)
     g_mutex_unlock (&self->lock);
     goto pause_task;
   }
+
+  /* normally we don't count buffers */
+  if (G_UNLIKELY (self->num_buffers_left >= 0)) {
+    GST_DEBUG_OBJECT (pad, "Buffers left %d", self->num_buffers_left);
+    if (self->num_buffers_left == 0) {
+      flow_ret = GST_FLOW_EOS;
+      g_mutex_unlock (&self->lock);
+      goto pause_task;
+    } else {
+      self->num_buffers_left--;
+    }
+  }
   buffer = gst_buffer_copy (self->buffer);
 
   g_mutex_unlock (&self->lock);
@@ -846,9 +921,7 @@ pause_task:
     } else if (flow_ret == GST_FLOW_NOT_LINKED || flow_ret < GST_FLOW_EOS) {
       GstEvent *e = gst_event_new_eos ();
 
-      GST_ELEMENT_ERROR (self, STREAM, FAILED,
-          ("Internal data stream error."),
-          ("stream stopped, reason %s", reason));
+      GST_ELEMENT_FLOW_ERROR (self, flow_ret);
 
       if (self->seqnum)
         gst_event_set_seqnum (e, self->seqnum);
