@@ -308,19 +308,22 @@ gst_interleave_channel_positions_to_mask (GValueArray * positions,
 static void
 gst_interleave_set_channel_positions (GstInterleave * self, GstStructure * s)
 {
-  guint64 channel_mask = 0;
-
-  if (self->channel_positions != NULL &&
+  if (self->channels <= 64 &&
+      self->channel_positions != NULL &&
       self->channels == self->channel_positions->n_values) {
     if (!gst_interleave_channel_positions_to_mask (self->channel_positions,
-            self->default_channels_ordering_map, &channel_mask)) {
+            self->default_channels_ordering_map, &self->channel_mask)) {
       GST_WARNING_OBJECT (self, "Invalid channel positions, using NONE");
-      channel_mask = 0;
+      self->channel_mask = 0;
     }
   } else {
-    GST_WARNING_OBJECT (self, "Using NONE channel positions");
+    self->channel_mask = 0;
+    if (self->channels <= 64) {
+      GST_WARNING_OBJECT (self, "Using NONE channel positions");
+    }
   }
-  gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK, channel_mask, NULL);
+  gst_structure_set (s, "channel-mask", GST_TYPE_BITMASK, self->channel_mask,
+      NULL);
 }
 
 static void
@@ -359,10 +362,8 @@ gst_interleave_class_init (GstInterleaveClass * klass)
       "Andy Wingo <wingo at pobox.com>, "
       "Sebastian Dröge <slomo@circular-chaos.org>");
 
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&sink_template));
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (gstelement_class, &sink_template);
+  gst_element_class_add_static_pad_template (gstelement_class, &src_template);
 
   /* Reference GstInterleavePad class to have the type registered from
    * a threadsafe context
@@ -1220,7 +1221,6 @@ gst_interleave_collected (GstCollectPads * pads, GstInterleave * self)
       goto next;
     }
     ncollected++;
-    gst_buffer_map (inbuf, &input_info, GST_MAP_READ);
 
     if (timestamp == -1)
       timestamp = GST_BUFFER_TIMESTAMP (inbuf);
@@ -1230,9 +1230,12 @@ gst_interleave_collected (GstCollectPads * pads, GstInterleave * self)
 
     empty = FALSE;
     channel = GST_INTERLEAVE_PAD_CAST (cdata->pad)->channel;
-    outdata =
-        write_info.data + width * self->default_channels_ordering_map[channel];
+    if (self->channels <= 64 && self->channel_mask) {
+      channel = self->default_channels_ordering_map[channel];
+    }
+    outdata = write_info.data + width * channel;
 
+    gst_buffer_map (inbuf, &input_info, GST_MAP_READ);
     self->func (outdata, input_info.data, self->channels, nsamples);
     gst_buffer_unmap (inbuf, &input_info);
 
