@@ -95,6 +95,12 @@ G_BEGIN_DECLS
 #define FOURCC_avc1     GST_MAKE_FOURCC('a','v','c','1')
 #define FOURCC_avc3     GST_MAKE_FOURCC('a','v','c','3')
 #define FOURCC_avcC     GST_MAKE_FOURCC('a','v','c','C')
+#define FOURCC_c608     GST_MAKE_FOURCC('c','6','0','8')
+#define FOURCC_c708     GST_MAKE_FOURCC('c','7','0','8')
+#define FOURCC_ccdp     GST_MAKE_FOURCC('c','c','d','p')
+#define FOURCC_cdat     GST_MAKE_FOURCC('c','d','a','t')
+#define FOURCC_cdt2     GST_MAKE_FOURCC('c','d','t','2')
+#define FOURCC_clcp     GST_MAKE_FOURCC('c','l','c','p')
 #define FOURCC_clip     GST_MAKE_FOURCC('c','l','i','p')
 #define FOURCC_cmov     GST_MAKE_FOURCC('c','m','o','v')
 #define FOURCC_cmvd     GST_MAKE_FOURCC('c','m','v','d')
@@ -167,6 +173,7 @@ G_BEGIN_DECLS
 #define FOURCC_mhlr     GST_MAKE_FOURCC('m','h','l','r')
 #define FOURCC_minf     GST_MAKE_FOURCC('m','i','n','f')
 #define FOURCC_moov     GST_MAKE_FOURCC('m','o','o','v')
+#define FOURCC_mp3_     GST_MAKE_FOURCC('m','p','3',' ')
 #define FOURCC_mp4a     GST_MAKE_FOURCC('m','p','4','a')
 #define FOURCC_mp4s     GST_MAKE_FOURCC('m','p','4','s')
 #define FOURCC_mp4s     GST_MAKE_FOURCC('m','p','4','s')
@@ -258,6 +265,12 @@ G_BEGIN_DECLS
 #define FOURCC_wide     GST_MAKE_FOURCC('w','i','d','e')
 #define FOURCC_zlib     GST_MAKE_FOURCC('z','l','i','b')
 #define FOURCC_lpcm     GST_MAKE_FOURCC('l','p','c','m')
+#define FOURCC_av01     GST_MAKE_FOURCC('a','v','0','1')
+#define FOURCC_av1C     GST_MAKE_FOURCC('a','v','1','C')
+#define FOURCC_av1f     GST_MAKE_FOURCC('a','v','1','f')
+#define FOURCC_av1m     GST_MAKE_FOURCC('a','v','1','m')
+#define FOURCC_av1s     GST_MAKE_FOURCC('a','v','1','s')
+#define FOURCC_av1M     GST_MAKE_FOURCC('a','v','1','M')
 
 #define FOURCC_cfhd     GST_MAKE_FOURCC('C','F','H','D')
 #define FOURCC_ap4x     GST_MAKE_FOURCC('a','p','4','x')
