@@ -209,7 +209,7 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
    * skip the processing of streams that already contain the necessary tags.
    * Data will flow as usual but the element will not consume CPU time and will
    * not generate result tags.  To enable possible skipping, set this property
-   * to #FALSE.
+   * to %FALSE.
    *
    * If used in conjunction with <link linkend="GstRgAnalysis--num-tracks">album
    * processing</link>, the element will skip the number of remaining album
@@ -267,10 +267,8 @@ gst_rg_analysis_class_init (GstRgAnalysisClass * klass)
   trans_class->stop = GST_DEBUG_FUNCPTR (gst_rg_analysis_stop);
   trans_class->passthrough_on_same_caps = TRUE;
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_factory));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_factory));
+  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_static_pad_template (element_class, &sink_factory);
   gst_element_class_set_static_metadata (element_class, "ReplayGain analysis",
       "Filter/Analyzer/Audio",
       "Perform the ReplayGain analysis",
