@@ -205,7 +205,7 @@ gst_aspect_ratio_crop_finalize (GObject * object)
   aspect_ratio_crop = GST_ASPECT_RATIO_CROP (object);
 
   g_mutex_clear (&aspect_ratio_crop->crop_lock);
-  gst_caps_replace (&aspect_ratio_crop->renegotiation_caps, NULL);
+  gst_clear_caps (&aspect_ratio_crop->renegotiation_caps);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -486,7 +486,7 @@ gst_aspect_ratio_crop_set_property (GObject * object, guint prop_id,
 
   if (recheck) {
     GST_OBJECT_LOCK (aspect_ratio_crop);
-    gst_caps_replace (&aspect_ratio_crop->renegotiation_caps, NULL);
+    gst_clear_caps (&aspect_ratio_crop->renegotiation_caps);
     aspect_ratio_crop->renegotiation_caps =
         gst_pad_get_current_caps (aspect_ratio_crop->sink);
     GST_OBJECT_UNLOCK (aspect_ratio_crop);
