@@ -64,6 +64,8 @@ struct _GstV4l2BufferPool
   gboolean empty;
   GCond empty_cond;
 
+  gboolean orphaned;
+
   GstV4l2Allocator *vallocator;
   GstAllocator *allocator;
   GstAllocationParams params;
@@ -77,6 +79,7 @@ struct _GstV4l2BufferPool
   guint min_latency;         /* number of buffers we will hold */
   guint max_latency;         /* number of buffers we can hold */
   guint num_queued;          /* number of buffers queued in the driver */
+  guint num_allocated;       /* number of buffers allocated */
   guint copy_threshold;      /* when our pool runs lower, start handing out copies */
 
   gboolean streaming;
@@ -109,6 +112,8 @@ void                gst_v4l2_buffer_pool_copy_at_threshold (GstV4l2BufferPool *
 
 gboolean            gst_v4l2_buffer_pool_flush   (GstBufferPool *pool);
 
+gboolean            gst_v4l2_buffer_pool_orphan  (GstBufferPool ** pool);
+
 G_END_DECLS
 
 #endif /*__GST_V4L2_BUFFER_POOL_H__ */
