@@ -20,9 +20,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#include <unistd.h>
-#include <math.h>
-
 /* suppress warnings for deprecated API such as GValueArray
  * with newer GLib versions (>= 2.31.0) */
 #define GLIB_DISABLE_DEPRECATION_WARNINGS
@@ -96,6 +93,7 @@ cleanup_level (GstElement * level)
 {
   GST_DEBUG ("cleanup_level");
 
+  gst_check_drop_buffers ();
   gst_pad_set_active (mysrcpad, FALSE);
   gst_pad_set_active (mysinkpad, FALSE);
   gst_check_teardown_src_pad (level);
@@ -152,7 +150,8 @@ GST_START_TEST (test_ref_counts)
   GstMessage *message;
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 10, NULL);
   fail_unless (gst_element_set_state (level,
           GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
       "could not set to playing");
@@ -193,7 +192,6 @@ GST_START_TEST (test_ref_counts)
   gst_element_set_bus (level, NULL);
   ASSERT_OBJECT_REFCOUNT (bus, "bus", 1);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   fail_unless (gst_element_set_state (level,
           GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS, "could not set to null");
   ASSERT_OBJECT_REFCOUNT (level, "level", 1);
@@ -212,7 +210,8 @@ GST_START_TEST (test_message_is_valid)
   GstClockTime endtime, ts, duration;
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 10, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -256,7 +255,8 @@ GST_START_TEST (test_int16)
   const gchar *fields[3] = { "rms", "peak", "decay" };
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 10, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -294,7 +294,6 @@ GST_START_TEST (test_int16)
   gst_message_unref (message);
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
@@ -308,13 +307,14 @@ GST_START_TEST (test_int16_panned)
   GstBus *bus;
   GstMessage *message;
   const GstStructure *structure;
-  gint j;
+  gint i, j;
   const GValue *list, *value;
   gdouble dB;
   const gchar *fields[3] = { "rms", "peak", "decay" };
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 30, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -328,44 +328,50 @@ GST_START_TEST (test_int16_panned)
   fail_if ((outbuffer = (GstBuffer *) buffers->data) == NULL);
   fail_unless (inbuffer == outbuffer);
 
-  message = gst_bus_poll (bus, GST_MESSAGE_ELEMENT, -1);
-  structure = gst_message_get_structure (message);
+  /* do multiple messages per buffer, to verify that the inner loop in level
+   * advances the read-index correctly, see
+   * https://bugzilla.gnome.org/show_bug.cgi?id=754144
+   */
+  for (i = 0; i < 3; i++) {
+    GST_DEBUG ("get message number %d", i);
+    message = gst_bus_poll (bus, GST_MESSAGE_ELEMENT, -1);
+    structure = gst_message_get_structure (message);
 
-  /* silence has 0 dB for rms, peak and decay */
-  for (j = 0; j < 3; ++j) {
-    GValueArray *arr;
+    /* silence has 0 dB for rms, peak and decay */
+    for (j = 0; j < 3; ++j) {
+      GValueArray *arr;
 
-    list = gst_structure_get_value (structure, fields[j]);
-    arr = g_value_get_boxed (list);
-    value = g_value_array_get_nth (arr, 0);
-    dB = g_value_get_double (value);
-    GST_DEBUG ("%s[0] is %lf", fields[j], dB);
+      list = gst_structure_get_value (structure, fields[j]);
+      arr = g_value_get_boxed (list);
+      value = g_value_array_get_nth (arr, 0);
+      dB = g_value_get_double (value);
+      GST_DEBUG ("%s[0] is %lf", fields[j], dB);
 #ifdef HAVE_ISINF
-    fail_unless (isinf (dB));
+      fail_unless (isinf (dB));
 #elif defined (HAVE_FPCLASS)
-    fail_unless (fpclass (dB) == FP_NINF);
+      fail_unless (fpclass (dB) == FP_NINF);
 #endif
-  }
-  /* block wave of half amplitude has -5.94 dB for rms, peak and decay */
-  for (j = 0; j < 3; ++j) {
-    GValueArray *arr;
+    }
+    /* block wave of half amplitude has -5.94 dB for rms, peak and decay */
+    for (j = 0; j < 3; ++j) {
+      GValueArray *arr;
 
-    list = gst_structure_get_value (structure, fields[j]);
-    arr = g_value_get_boxed (list);
-    value = g_value_array_get_nth (arr, 1);
-    dB = g_value_get_double (value);
-    GST_DEBUG ("%s[1] is %lf", fields[j], dB);
-    fail_if (dB < -6.1);
-    fail_if (dB > -5.9);
+      list = gst_structure_get_value (structure, fields[j]);
+      arr = g_value_get_boxed (list);
+      value = g_value_array_get_nth (arr, 1);
+      dB = g_value_get_double (value);
+      GST_DEBUG ("%s[1] is %lf", fields[j], dB);
+      fail_if (dB < -6.1);
+      fail_if (dB > -5.9);
+    }
+    gst_message_unref (message);
   }
 
   /* clean up */
   /* flush current messages,and future state change messages */
   gst_bus_set_flushing (bus, TRUE);
-  gst_message_unref (message);
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
@@ -385,7 +391,8 @@ GST_START_TEST (test_float)
   const gchar *fields[3] = { "rms", "peak", "decay" };
 
   level = setup_level (LEVEL_F32_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 10, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 10, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -423,7 +430,6 @@ GST_START_TEST (test_float)
   gst_message_unref (message);
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
@@ -443,7 +449,8 @@ GST_START_TEST (test_message_on_eos)
   gdouble dB;
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 5, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 5, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -489,7 +496,6 @@ GST_START_TEST (test_message_on_eos)
   gst_message_unref (message);
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
@@ -504,7 +510,8 @@ GST_START_TEST (test_message_count)
   GstMessage *message;
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 20, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 20, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -528,7 +535,6 @@ GST_START_TEST (test_message_count)
 
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
@@ -545,7 +551,8 @@ GST_START_TEST (test_message_timestamps)
   GstClockTime ts1, dur1, ts2;
 
   level = setup_level (LEVEL_S16_CAPS_STRING);
-  g_object_set (level, "message", TRUE, "interval", GST_SECOND / 20, NULL);
+  g_object_set (level, "post-messages", TRUE,
+      "interval", (guint64) GST_SECOND / 20, NULL);
   gst_element_set_state (level, GST_STATE_PLAYING);
   /* create a bus to get the level message on */
   bus = gst_bus_new ();
@@ -575,7 +582,6 @@ GST_START_TEST (test_message_timestamps)
 
   gst_element_set_bus (level, NULL);
   gst_object_unref (bus);
-  gst_buffer_unref (outbuffer);
   gst_element_set_state (level, GST_STATE_NULL);
   cleanup_level (level);
 }
