@@ -138,6 +138,7 @@ GST_START_TEST (test_rtxsend_rtxreceive)
   GstBuffer *inbufs[5];
   GstHarness *hrecv = gst_harness_new ("rtprtxreceive");
   GstHarness *hsend = gst_harness_new ("rtprtxsend");
+  gint i;
 
   pt_map = gst_structure_new ("application/x-rtp-pt-map",
       "96", G_TYPE_UINT, rtx_pt, NULL);
@@ -154,7 +155,7 @@ GST_START_TEST (test_rtxsend_rtxreceive)
       "encoding-name = (string)RAW");
 
   /* Push 'packets_num' packets through rtxsend to rtxreceive */
-  for (gint i = 0; i < packets_num; ++i) {
+  for (i = 0; i < packets_num; ++i) {
     inbufs[i] = create_rtp_buffer (master_ssrc, master_pt, 100 + i);
     gst_harness_push (hsend, gst_buffer_ref (inbufs[i]));
     gst_harness_push (hrecv, gst_harness_pull (hsend));
@@ -169,7 +170,7 @@ GST_START_TEST (test_rtxsend_rtxreceive)
      Push RTX packets from rtxsend to rtxreceive and
      check that the packet produced out of RTX packet is the same
      as an original packet */
-  for (gint i = 0; i < packets_num; ++i) {
+  for (i = 0; i < packets_num; ++i) {
     GstBuffer *outbuf;
     gst_harness_push_upstream_event (hrecv,
         create_rtx_event (master_ssrc, master_pt, 100 + i));
@@ -221,6 +222,7 @@ GST_START_TEST (test_rtxsend_rtxreceive_with_packet_loss)
   GstStructure *pt_map;
   GstHarness *hrecv = gst_harness_new ("rtprtxreceive");
   GstHarness *hsend = gst_harness_new ("rtprtxsend");
+  gint drop_nth_packet, i;
 
   pt_map = gst_structure_new ("application/x-rtp-pt-map",
       "96", G_TYPE_UINT, rtx_pt, NULL);
@@ -244,8 +246,8 @@ GST_START_TEST (test_rtxsend_rtxreceive_with_packet_loss)
   /* Push 'packets_num' packets through rtxsend to rtxreceive loosing every
      'drop_every_n_packets' packet. When we loose the packet we send RTX event
      through rtxreceive to rtxsend, and verify the packet was retransmitted */
-  for (gint drop_nth_packet = 2; drop_nth_packet < 10; ++drop_nth_packet) {
-    for (gint i = 0; i < packets_num; ++i, ++seqnum) {
+  for (drop_nth_packet = 2; drop_nth_packet < 10; ++drop_nth_packet) {
+    for (i = 0; i < packets_num; ++i, ++seqnum) {
       GstBuffer *outbuf;
       GstBuffer *inbuf = create_rtp_buffer (master_ssrc, master_pt, seqnum);
       gboolean drop_this_packet = ((i + 1) % drop_nth_packet) == 0;
@@ -326,8 +328,9 @@ create_rtxsenders (RtxSender * senders, guint senders_num)
 {
   GstStructure *recv_pt_map =
       gst_structure_new_empty ("application/x-rtp-pt-map");
+  gint i;
 
-  for (gint i = 0; i < senders_num; ++i) {
+  for (i = 0; i < senders_num; ++i) {
     gchar *master_pt_str;
     gchar *master_caps_str;
     GstStructure *send_pt_map;
@@ -366,7 +369,9 @@ static guint
 check_rtxsenders_stats_and_teardown (RtxSender * senders, guint senders_num)
 {
   guint total_pakets_num = 0;
-  for (gint i = 0; i < senders_num; ++i) {
+  gint i;
+
+  for (i = 0; i < senders_num; ++i) {
     guint rtx_requests;
     guint rtx_packets;
     g_object_get (G_OBJECT (senders[i].h->element),
@@ -390,6 +395,7 @@ GST_START_TEST (test_multi_rtxsend_rtxreceive_with_packet_loss)
   RtxSender senders[5];
   GstStructure *pt_map;
   GstHarness *hrecv = gst_harness_new ("rtprtxreceive");
+  gint drop_nth_packet, i, j;
 
   pt_map = create_rtxsenders (senders, 5);
   g_object_set (hrecv->element, "payload-type-map", pt_map, NULL);
@@ -410,8 +416,8 @@ GST_START_TEST (test_multi_rtxsend_rtxreceive_with_packet_loss)
      We need to make sure that all other senders will ignore the RTX event they
      can't act upon.
    */
-  for (gint drop_nth_packet = 2; drop_nth_packet < 5; ++drop_nth_packet) {
-    for (gint i = 0; i < total_pakets_num; ++i) {
+  for (drop_nth_packet = 2; drop_nth_packet < 5; ++drop_nth_packet) {
+    for (i = 0; i < total_pakets_num; ++i) {
       RtxSender *sender = &senders[i % senders_num];
       gboolean drop_this_packet = ((i + 1) % drop_nth_packet) == 0;
       GstBuffer *outbuf, *inbuf;
@@ -432,7 +438,7 @@ GST_START_TEST (test_multi_rtxsend_rtxreceive_with_packet_loss)
         rtxevent = gst_harness_pull_upstream_event (hrecv);
 
         /* ... to all the senders */
-        for (gint j = 0; j < senders_num; ++j)
+        for (j = 0; j < senders_num; ++j)
           gst_harness_push_upstream_event (senders[j].h,
               gst_event_ref (rtxevent));
         gst_event_unref (rtxevent);
@@ -457,7 +463,7 @@ GST_START_TEST (test_multi_rtxsend_rtxreceive_with_packet_loss)
          We should not have any packets in the harness queue by this point. It
          means our senders didn't produce the packets for the unknown RTX event.
        */
-      for (gint j = 0; j < senders_num; ++j)
+      for (j = 0; j < senders_num; ++j)
         fail_unless_equals_int (gst_harness_buffers_in_queue (senders[j].h), 0);
 
       ++sender->seqnum;
@@ -506,6 +512,7 @@ test_rtxsender_packet_retention (gboolean test_with_time)
       "96", G_TYPE_UINT, rtx_pt, NULL);
   GstStructure *ssrc_map = gst_structure_new ("application/x-rtp-ssrc-map",
       "1234567", G_TYPE_UINT, rtx_ssrc, NULL);
+  gint i, j;
 
   h = gst_harness_new ("rtprtxsend");
 
@@ -525,9 +532,9 @@ test_rtxsender_packet_retention (gboolean test_with_time)
       "encoding-name = (string)RAW");
 
   /* Now push all buffers and request retransmission every time for all of them */
-  for (gint i = 0; i < num_buffers; ++i, timestamp += timestamp_delta) {
+  for (i = 0; i < num_buffers; ++i, timestamp += timestamp_delta) {
     /* Request to retransmit all the previous ones */
-    for (gint j = 0; j < i; ++j) {
+    for (j = 0; j < i; ++j) {
       guint rtx_seqnum = 0x100 + j;
       gst_harness_push_upstream_event (h,
           create_rtx_event (master_ssrc, master_pt, rtx_seqnum));
@@ -565,6 +572,86 @@ GST_START_TEST (test_rtxsender_max_size_time)
 
 GST_END_TEST;
 
+static void
+test_rtxqueue_packet_retention (gboolean test_with_time)
+{
+  guint ssrc = 1234567;
+  guint pt = 96;
+  gint num_buffers = test_with_time ? 30 : 10;
+  gint half_buffers = num_buffers / 2;
+  GstClockTime timestamp_delta = GST_SECOND / 30;
+  GstClockTime timestamp = 0;
+  GstBuffer *buf;
+  GstHarness *h;
+  gint i, j;
+
+  h = gst_harness_new ("rtprtxqueue");
+
+  /* In both cases we want the rtxqueue to store 'half_buffers'
+     amount of buffers at most. In max-size-packets mode, it's trivial.
+     In max-size-time mode, we specify almost half a second, which is
+     the equivalent of 15 frames in a 30fps video stream.
+   */
+  g_object_set (h->element,
+      "max-size-packets", test_with_time ? 0 : half_buffers,
+      "max-size-time", test_with_time ? 498 : 0, NULL);
+
+  gst_harness_set_src_caps_str (h, "application/x-rtp, "
+      "media = (string)video, payload = (int)96, "
+      "ssrc = (uint)1234567, clock-rate = (int)90000, "
+      "encoding-name = (string)RAW");
+
+  /* Now push all buffers and request retransmission every time for all of them.
+   * Note that rtprtxqueue sends retransmissions in chain(), just before
+   * pushing out the chained buffer, a differentiation from rtprtxsend above
+   */
+  for (i = 0; i < num_buffers; ++i, timestamp += timestamp_delta) {
+    /* Request to retransmit all the previous ones */
+    for (j = 0; j < i; ++j) {
+      guint rtx_seqnum = 0x100 + j;
+      gst_harness_push_upstream_event (h,
+          create_rtx_event (ssrc, pt, rtx_seqnum));
+    }
+
+    /* push one packet */
+    buf = create_rtp_buffer (ssrc, pt, 0x100 + i);
+    GST_BUFFER_TIMESTAMP (buf) = timestamp;
+    gst_harness_push (h, buf);
+
+    /* Pull the ones supposed to be retransmitted */
+    for (j = 0; j < i; ++j) {
+      guint rtx_seqnum = 0x100 + j;
+      if (j >= i - half_buffers)
+        pull_and_verify (h, FALSE, ssrc, pt, rtx_seqnum);
+    }
+
+    /* There should be only one packet remaining in the queue now */
+    fail_unless_equals_int (gst_harness_buffers_in_queue (h), 1);
+
+    /* pull the one that we just pushed (comes after the retransmitted ones) */
+    pull_and_verify (h, FALSE, ssrc, pt, 0x100 + i);
+
+    /* Check there no extra buffers in the harness queue */
+    fail_unless_equals_int (gst_harness_buffers_in_queue (h), 0);
+  }
+
+  gst_harness_teardown (h);
+}
+
+GST_START_TEST (test_rtxqueue_max_size_packets)
+{
+  test_rtxqueue_packet_retention (FALSE);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (test_rtxqueue_max_size_time)
+{
+  test_rtxqueue_packet_retention (TRUE);
+}
+
+GST_END_TEST;
+
 static Suite *
 rtprtx_suite (void)
 {
@@ -580,6 +667,8 @@ rtprtx_suite (void)
   tcase_add_test (tc_chain, test_multi_rtxsend_rtxreceive_with_packet_loss);
   tcase_add_test (tc_chain, test_rtxsender_max_size_packets);
   tcase_add_test (tc_chain, test_rtxsender_max_size_time);
+  tcase_add_test (tc_chain, test_rtxqueue_max_size_packets);
+  tcase_add_test (tc_chain, test_rtxqueue_max_size_time);
 
   return s;
 }
